
"""
A collection of helper functions/tools I made in order to speed up my development process.

Keep in mind of these are old code that is quite shit so bare with me. (Plans to update them are coming)

I also tried documenting them as much as I could but due to time constraints, its not properly documented.
"""

from .logging import Logger
from .tools import *
from .osInterface import *
from flask import *
from .jarvis.api import API
from .database import Database
